"""
	Modul koji sadri klasu OsnovnaHTMLObrada i Veza.
	
	Klasa OsnovnaHTMLObrada ralanjuje .html fajl u string delove koje	stavlja
	u niz "delovi". Kad naie na komentar: <!--  Pocetak --> pa do komentara:
	<!-- Kraj -->, oznake: <tr>, <td>, <a>, </tr>, </td>, </a> kao i tekst izmeu
	oznaka <a> i </a> se stavljaju u objekte klase Veza a posle i u niz "veze"
	
	Metoda output vraa HTML dokument ponovo sastavljen od delia iz nizova "delovi" i "veze"
	
	<Pisano sa jEdit 4.1 final>
"""

__author__ = "Zlatan Kadragic (kadraga@ptt.yu)"
__version__ = "1.2"

from sgmllib import SGMLParser
import htmlentitydefs, sys

class Veza:
	POCETAK = '''        <tr>
          <td background="%s/fade.gif">
                    &middot; <a href="''' % sys.path[0]
	KRAJ = """</a>
          </td>
        </tr>\n"""
	
	def __init__(self, veza = '', ime = ''):
		self.staviVezu(veza)
		self.staviIme(ime)
		
	def staviVezu(self, s):
		self.link =  s
		
	def dajVezu(self):
		return self.link
		
	def staviIme(self, s):
		self.naziv = s.capitalize()
		
	def dajIme(self):
		return self.naziv
		
	def __str__(self):
		return Veza.POCETAK + self.link + '">' + self.naziv + Veza.KRAJ
		
	def __cmp__(self, drugi):
		if isinstance(drugi, Veza): # da li je drugi obj. instanca klase Slog
			if self.naziv == drugi.naziv: return 0 # jednaki su
			if self.naziv > drugi.naziv: return 1  # tekuci obj. je >
		return -1 # tekuci je < ili drugi obj. nije klase Slog (npr. None)


class OsnovnaHTMLObrada(SGMLParser):
	def reset(self):
		# proirenje (poziva je SGMLParser.__init__)
		self.delovi = []
		self.veze = []
		self.pocetakObrade = False
		self.pocetnaTacka = 0
		self.obradiOznacenuRec = False
		SGMLParser.reset(self)
		
	def unknown_starttag(self, oznaka, osobine):
		# poziva se za svaku otvarajuu oznaku (tag), ako ne postoji "start_imeOznake" metoda
		# osobine je niz parova (osobina, vrednost) koji predstavljaju osobine (atribute) oznake
		# npr. <a href="http://www.elitesecurity.org/"> :
		# oznaka="a", osobine=[("href", "http://www.elitesecurity.org/")]
		if not self.pocetakObrade:
			strOsobine = "".join([' %s="%s"' % (osobina, vrednost) for osobina, vrednost in osobine])
			self.delovi.append("<%(oznaka)s%(strOsobine)s>" % locals())
			
	def unknown_endtag(self, oznaka):
		# poziva se za svaku zatvarajuu oznaku, npr. </pre>, oznaka je "pre"
		if not self.pocetakObrade:
			self.delovi.append("</%(oznaka)s>" % locals())
			
	def handle_charref(self, brOpis):
		# poziva se za svaki brojani opis znaka (character reference), npr. "&#160;", brOpis je "160"
		if not self.pocetakObrade:
			self.delovi.append("&#%(brOpis)s;" % locals())
			
	def handle_entityref(self, znOpis):
		# poziva se za svaki znakovni opis nekog znaka (entity reference), npr. "&copy;", znOpis je "copy"
		if not self.pocetakObrade:
			self.delovi.append("&%(znOpis)s" % locals())
			# usvojeni HTML znakovni opisi se zatvaraju sa ";" a drugi ne
			if htmlentitydefs.entitydefs.has_key(znOpis):
				self.delovi.append(";")
				
	def handle_data(self, tekst):
		# poziva se za svaki tekst izmeu oznaka u HTML kodu
		if self.obradiOznacenuRec:
			self.veze[-1].staviIme(tekst)
		else:
			if not self.pocetakObrade:
				self.delovi.append(tekst)
			else:  # ako obrauje da ne ubacuje prazne str-ove oko tr i td oznaka
				if tekst.strip(): # True ako nije prazan str
					self.delovi.append(tekst)
					
	def handle_comment(self, tekst):
		# poziva se za svaki HTML komentar, e.g. <!-- poetak Javascript koda -->
		if not self.pocetakObrade:
			self.delovi.append("<!--%(tekst)s-->" % locals())
		if tekst.find('Pocetak') != -1:
			self.pocetakObrade = True
			self.pocetnaTacka = len(self.delovi)
		elif tekst.find('Kraj') != -1:
			self.pocetakObrade = False
			self.delovi.append("<!--%(tekst)s-->" % locals())
			
	def handle_pi(self, tekst):
		# poziva se za svaku HTML naredbu za obradu, npr. <?naredba>
		if not self.pocetakObrade:
			self.delovi.append("<?%(tekst)s>" % locals())
		
	def handle_decl(self, tekst):
		# poziva se za svaki XML DOCTYPE, npr.
		# <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
		#	 "http://www.w3.org/TR/html4/loose.dtd">
		if not self.pocetakObrade:
			self.delovi.append("<!%(tekst)s>" % locals())
		
	def start_tr(self, osobine):
		# poziva se za svaku <tr osobina = "vrednost"> oznaku u HTML kodu
		if not self.pocetakObrade:
			strOsobine = "".join([' %s="%s"' % (osobina, vrednost) for osobina, vrednost in osobine])
			self.delovi.append("<tr%(strOsobine)s>" % locals())
			
	def end_tr(self):
		# poziva se za svaku </tr> oznaku u HTML kodu
		if not self.pocetakObrade:
			self.delovi.append("</tr>")
			
	def start_td(self, osobine):
		# poziva se za svaku <td osobina = "vrednost"> oznaku u HTML kodu
		if not self.pocetakObrade:
			strOsobine = "".join([' %s="%s"' % (osobina, vrednost) for osobina, vrednost in osobine])
			self.delovi.append("<td%(strOsobine)s>" % locals())

	def end_td(self):
		# poziva se za svaku </td> oznaku u HTML kodu
		if not self.pocetakObrade:
			self.delovi.append("</td>")
			
	def start_a(self, osobine):
		# poziva se za svaku <a href = "neto"> oznaku u HTML kodu
		if self.pocetakObrade:
			self.obradiOznacenuRec = True
			self.veze.append( Veza(osobine[0][1]) )
		else:
			strOsobine = "".join([' %s="%s"' % (osobina, vrednost) for osobina, vrednost in osobine])
			self.delovi.append("<a%(strOsobine)s>" % locals())
		
	def end_a(self):
		# poziva se za svaku </a> oznaku u HTML kodu
		self.obradiOznacenuRec = False
		if not self.pocetakObrade:
			self.delovi.append('</a>')
		
	def output(self):
		umetak = ['\n'] + [str(e) for e in self.veze]
				
		return ''.join(self.delovi[: self.pocetnaTacka] + umetak + self.delovi[self.pocetnaTacka :])

################# KRAJ KLASE OsnovnaHTMLObrada

def sortirajLinkove(fajl):
	try:
		ulaz = file(fajl, "rb")
		try:
			htmlIzvor = ulaz.read()
		finally:
			ulaz.close()
	except IOError:
		print >> sys.stderr, "Greska u citanju. Ne postoji fajl:", fajl
		sys.exit(1)
	
	parser = OsnovnaHTMLObrada()
	parser.feed(htmlIzvor)
	parser.close()
	parser.veze.sort() # sortira azbuno veze
	return parser.output()
	
def test(fajl):
	tekst = sortirajLinkove(fajl)
	import os
	os.rename(fajl, fajl + '.old')
	izlaz = file(fajl, "wb")
	izlaz.write(tekst)
	izlaz.close()
	import webbrowser
	webbrowser.open_new(fajl)
	
if __name__ == "__main__":
	test(r"D:\Biblioteka\Python\diveintopy\py\index.html")
